// analyze consumption dynamics in NK_BW, NK, and RBC
// in "Where is the trend the cycle?" by
// Jean-Paul L'Huillier and Donghoon Yoo
// September 2018

var tfp x z dtfp                                                           // common productivity process

ySOE_RBC cSOE_RBC nSOE_RBC iSOE_RBC casSOE_RBC kSOE_RBC rSOE_RBC bSOE_RBC caSOE_RBC lamSOE_RBC psiSOE_RBC cySOE_RBC iySOE_RBC iaSOE_RBC yaSOE_RBC kaSOE_RBC   
dySOE_RBC dcSOE_RBC dnSOE_RBC                                              // model I: SOE-RBC

lamNK_BW psiNK_BW cNK_BW iNK_BW kbarNK_BW kkNK_BW uNK_BW yNK_BW nNK_BW rkNK_BW wNK_BW piNK_BW mcNK_BW mc_wNK_BW riNK_BW caNK_BW iaNK_BW yaNK_BW kaNK_BW waNK_BW cyNK_BW rNK_BW 
dcNK_BW dyNK_BW dnNK_BW diNK_BW dy8NK_BW                                // model II: NK_BW

c r b cas ca                                                               // model III: SOE-RBC (absent K and I)

caNK iNK piNK yNK dyNK lamNK dy8NK                                  // model IV: NK

k_RBC i_RBC y_RBC n_RBC c_RBC lam_RBC r_RBC w_RBC                          // model V: RBC
ca_RBC ya_RBC ka_RBC ia_RBC wa_RBC dy_RBC dc_RBC dn_RBC di_RBC dy8_RBC  ; 

// these are normalized (unit variance) tech shocks
varexo eps eta;

parameters rho_x rho_z sig_eps sig_eta                                     // productivity process 
           psi bet del nu alp phi                                          // common parameters
           CY IY KY NXY CY1                                                // steady states in SOE-RBC
           CY_RBC IY_RBC KY_RBC                                            // steady states in RBC
           RkP WAP KAN IYNK_BW RkKPY                                       // steady states in NK_BW
           calNK gam_piNK h chi xi calNK_BW cal_w iot iot_w mu_p mu_w rho_r gam_piNK_BW gam_y phi_dy betNK_BW
           betNK;                                                          // parameters in NK_BW and NK

// Parametrization           
// limiting=1 is ``parametrization I'' and limiting=0 corresponds to ``parametrization II''
@#define limiting =1                               
@#if limiting == 0
bet=0.99999;                                                               // discount rate (SOE-RBC model)
psi = 0.000000000001;                                                      // elasticity of interest rate
BY=0;                                                                      // steady state level of normalized debt
h=0;                                                                       // consumption habit
calNK_BW = 0.99999999;                                                     // Calvo price
gam_y = 0.000001;                                                          // Taylor rule output
phi_dy= 0;                                                                 // Taylor rule output growth
@# else
bet=1/(1+0.0013);
psi=0.001;
BY=0.1;
h=0.78;
calNK_BW = 0.84;
gam_y = 0.07;
phi_dy= 0.24;
@# endif

// Parameters ommon to both models
alp = .17;                                  // capital share
phi = 3.79;                                 // invese Frisch elasticity
xi = 5.30;                                  // elasticity of capital utilization
nu = 2.85;                                  // investment adjustment costs (SOE-RBC model)
chi = 2.85;                                 // investment adjustment costs (NK-BW model)

// Shock processes
rho_x = 0.3;                                // persistence permanent shock                
rho_z = 0.3;                                // persistence transitor shock
sig_eps = 1;                                // standard dev. permanent shock
sig_eta = 1;                                // standard dev. temporary shock

// NK-BW model parameters
del = .025;                                                                  
betNK_BW=bet         ;                      // discount rate (NK-BW model)
cal_w = 0.70;                               // Calvo wages
mu_p = .23;                                 // price markup
mu_w = .15;                                 // wage markup
gam_piNK_BW = 2.09;                         // Taylor rule inflation
iot = .24;                                  // price indexation
iot_w = .11;                                // wage indexation
rho_r = .82;                                // persistence nominal interest rate

// NK model parameters
calNK = .84;                                // Calvo price (NK model)
gam_piNK = 2.09;                            // Taylor rule inflation (NK model)
betNK=bet;

// Steady state relations SOE-RBC model
NXY=(1-bet)*BY;                             // once BY is chosen, NXY and CY can also be determined
IY = del*alp/((1/bet)-(1-del)); 
CY=1-NXY-IY;
CY1 =1-NXY;
KY = IY/del;

// Steady state relations RBC model
KY_RBC=alp/((1/bet)-(1-del));
IY_RBC=del*KY_RBC;
CY_RBC=1-IY_RBC;

// Steady state relations NK-BW model
RkP=1/betNK_BW -(1-del); 
WAP=((alp^alp*(1-alp)^(1-alp)/(1+mu_p))*RkP^(-alp))^(1/(1-alp));
KAN=(WAP/RkP)*(alp/(1-alp));
IYNK_BW=del*(KAN)^(1-alp);
RkKPY = RkP*KAN^(1-alp);

model;
// productivity: 4 eqs
tfp = x + z;
x - x(-1) = rho_x*(x(-1)-x(-2)) + sig_eps*eps;
z = rho_z*z(-1) + sig_eta*eta;
dtfp = tfp - tfp(-1);

// RBC: 16 eqs
k_RBC=(1-del)*(k_RBC(-1)-dtfp)+(IY_RBC/KY_RBC)*i_RBC; 
y_RBC=(1-alp)*n_RBC+alp*k_RBC(-1)-alp*dtfp; 
CY_RBC*(c_RBC)+IY_RBC*(i_RBC)=y_RBC;                                  
lam_RBC=(1+phi)*n_RBC-y_RBC;                                    
lam_RBC=-c_RBC; 
lam_RBC=lam_RBC(+1)-dtfp(+1)+r_RBC;  
w_RBC=alp*k_RBC(-1)-alp*n_RBC-alp*dtfp;
r_RBC=(alp-1)*k_RBC(-1)+(1-alp)*n_RBC+(1-alp)*dtfp;
ca_RBC = c_RBC + tfp;                                           
ia_RBC = i_RBC + tfp;
ya_RBC = y_RBC + tfp;
ka_RBC = k_RBC + tfp;
wa_RBC = w_RBC + tfp;
dy_RBC=y_RBC-y_RBC(-1);
dc_RBC=c_RBC-c_RBC(-1);
dn_RBC=n_RBC-n_RBC(-1);
di_RBC=i_RBC-i_RBC(-1);
dy8_RBC=y_RBC-y_RBC(-8);
//dy_RBC=ya_RBC-ya_RBC(-1);
//dc_RBC=ca_RBC-ca_RBC(-1);
//dn_RBC=n_RBC-n_RBC(-1);
//di_RBC=ia_RBC-ia_RBC(-1);

// Cao and L'huillier (2014): 5 eqs
c=c(+1)+dtfp(+1)-r;
r=psi*b;
c+(1/CY1)*cas=0;
cas=b(-1)-bet*b+((1-CY1)/(1-bet))*(bet*r-dtfp);
ca=c+tfp;

// NK: 5 eqs
lamNK = (h*betNK_BW/((1-h*betNK_BW)*(1-h)))*yNK(+1) - ((1+h^2*betNK_BW)/((1-h*betNK_BW)*(1-h)))*yNK + (h/((1-h*betNK_BW)*(1-h)))*yNK(-1) + (h*betNK_BW/((1-h*betNK_BW)*(1-h)))*dtfp(+1) - (h/((1-h*betNK_BW)*(1-h)))*dtfp;
lamNK = lamNK(+1) - dtfp(+1) + iNK - piNK(+1);
piNK =  betNK_BW*piNK(+1) + ((1+phi)*(1-calNK*betNK_BW)*(1-calNK)/calNK)*(caNK - tfp);
iNK = gam_piNK*piNK;
yNK =caNK-tfp;
dyNK=caNK-caNK(-1);
dy8NK=caNK-caNK(-8);

// SOE_RBC: 19 eqs
ySOE_RBC=(1-alp)*nSOE_RBC+alp*kSOE_RBC(-1)-alp*dtfp;                                           // production (end of period)
CY*(cSOE_RBC-ySOE_RBC)+IY*(iSOE_RBC-ySOE_RBC)+casSOE_RBC=0;                                    // resource constraint 
CY*(cSOE_RBC-ySOE_RBC)+IY*(iSOE_RBC-ySOE_RBC)=bet*bSOE_RBC-bSOE_RBC(-1)+((1-CY-IY)/(1-bet))*(dtfp+ySOE_RBC-ySOE_RBC(-1)-bet*rSOE_RBC);
                                                                                               // budget constraint (foc wrt lamCLY)
lamSOE_RBC = (h*bet/((1-h*bet)*(1-h)))*cSOE_RBC(+1) - ((1+h^2*bet)/((1-h*bet)*(1-h)))*cSOE_RBC + (h/((1-h*bet)*(1-h)))*cSOE_RBC(-1) + (h*bet/((1-h*bet)*(1-h)))*dtfp(+1) - (h/((1-h*bet)*(1-h)))*dtfp;
                                                                                               // for wrt C (habit)
lamSOE_RBC=(1+phi)*nSOE_RBC-ySOE_RBC;                                                          // foc wrt N
lamSOE_RBC=lamSOE_RBC(+1)-dtfp(+1)+rSOE_RBC;                                                   // foc wrt B
lamSOE_RBC=psiSOE_RBC;                                                                         // foc wrt I
rSOE_RBC=psi*bSOE_RBC;                                                                         // pricing of debt 
kSOE_RBC=(1-del)*(kSOE_RBC(-1)-dtfp)+(IY/KY)*iSOE_RBC;                                         // law of capital accumulation (wrt psiCLY) 
-(1+psiSOE_RBC+nu*kSOE_RBC-nu*kSOE_RBC(-1)+nu*dtfp)+alp*bet*(1/KY)*(1-kSOE_RBC+ySOE_RBC(+1)+lamSOE_RBC(+1))+bet*(1-del)*(1+psiSOE_RBC(+1)-dtfp(+1))-bet*(nu*kSOE_RBC-nu*kSOE_RBC(+1)-nu*dtfp(+1))=0;
                                                                                               // foc wrt K
caSOE_RBC = cSOE_RBC + tfp;                                                                    // consumption
iaSOE_RBC = iSOE_RBC + tfp;
yaSOE_RBC = ySOE_RBC + tfp;
kaSOE_RBC = kSOE_RBC + tfp;
cySOE_RBC=cSOE_RBC-ySOE_RBC;
iySOE_RBC=iSOE_RBC-ySOE_RBC;
dySOE_RBC=ySOE_RBC-ySOE_RBC(-1);
dcSOE_RBC=cSOE_RBC-cSOE_RBC(-1);
dnSOE_RBC=nSOE_RBC-nSOE_RBC(-1);

// NK_BW: 26 eqs
lamNK_BW = (h*betNK_BW/((1-h*betNK_BW)*(1-h)))*cNK_BW(+1) - ((1+h^2*betNK_BW)/((1-h*betNK_BW)*(1-h)))*cNK_BW + (h/((1-h*betNK_BW)*(1-h)))*cNK_BW(-1) + (h*betNK_BW/((1-h*betNK_BW)*(1-h)))*dtfp(+1) - (h/((1-h*betNK_BW)*(1-h)))*dtfp;
lamNK_BW = lamNK_BW(+1) - dtfp(+1) + riNK_BW;
riNK_BW = rNK_BW - piNK_BW(+1);
psiNK_BW = (1-del)*betNK_BW*(psiNK_BW(+1)-dtfp(+1)) + (1-(1-del)*betNK_BW)*(lamNK_BW(+1)-dtfp(+1)+rkNK_BW(+1));
lamNK_BW = psiNK_BW - chi*(iNK_BW-iNK_BW(-1)+dtfp) + betNK_BW*chi*(iNK_BW(+1)-iNK_BW+dtfp(+1));
rkNK_BW = xi*uNK_BW;
kkNK_BW = uNK_BW + kbarNK_BW(-1) - dtfp;
kbarNK_BW = (1-del)*(kbarNK_BW(-1)-dtfp) + del*iNK_BW;
yNK_BW = alp*kkNK_BW + (1-alp)*nNK_BW;                                                            
piNK_BW = (iot/(1+iot*betNK_BW))*piNK_BW(-1) + (betNK_BW/(1+iot*betNK_BW))*piNK_BW(+1) + (((1-calNK_BW*betNK_BW)*(1-calNK_BW))/(calNK_BW*(1+iot*betNK_BW)))*mcNK_BW;                                                            
mcNK_BW = alp*rkNK_BW + (1-alp)*wNK_BW;
kkNK_BW - nNK_BW = wNK_BW - rkNK_BW;
wNK_BW = (1/(1+betNK_BW))*wNK_BW(-1) + (betNK_BW/(1+betNK_BW))*wNK_BW(+1) + (iot_w/(1+betNK_BW))*piNK_BW(-1) - ((1+betNK_BW*iot_w)/(1+betNK_BW))*piNK_BW + (betNK_BW/(1+betNK_BW))*piNK_BW(+1) + (iot_w/(1+betNK_BW))*dtfp(-1) - ((1+betNK_BW*iot_w)/(1+betNK_BW))*dtfp + (betNK_BW/(1+betNK_BW))*dtfp(+1) - (((1-cal_w*betNK_BW)*(1-cal_w))/(cal_w*(1+betNK_BW)*(1+phi*(1+1/mu_w))))*mc_wNK_BW;
mc_wNK_BW = wNK_BW - phi*nNK_BW + lamNK_BW;
rNK_BW = rho_r*rNK_BW(-1) + (1-rho_r)*(gam_piNK_BW*piNK_BW+gam_y*yNK_BW) + phi_dy*(yNK_BW-yNK_BW(-1));
(1-IYNK_BW-RkKPY)*cNK_BW + (IY)*iNK_BW + RkKPY*uNK_BW = yNK_BW;
caNK_BW = cNK_BW + tfp;
iaNK_BW = iNK_BW + tfp;
yaNK_BW = yNK_BW + tfp;
kaNK_BW = kkNK_BW + tfp;
waNK_BW = wNK_BW + tfp;
cyNK_BW = cNK_BW - yNK_BW;
dcNK_BW=caNK_BW-caNK_BW(-1);
dyNK_BW=yaNK_BW-yaNK_BW(-1);
dy8NK_BW=yaNK_BW-yaNK_BW(-8);
diNK_BW=iaNK_BW-iaNK_BW(-1);
dnNK_BW=nNK_BW-nNK_BW(-1);
end;

shocks;
var eps; stderr 1;
var eta; stderr 1;
end;

stoch_simul(conditional_variance_decomposition = [1:20],IRF=20,nograph)  yaNK_BW iaNK_BW caNK_BW rNK_BW yNK_BW dnNK_BW y_RBC dcNK_BW dyNK_BW  dyNK yNK caNK tfp  r_RBC dy_RBC dc_RBC dn_RBC diNK_BW di_RBC ya_RBC ia_RBC  dy8NK dy8NK_BW dy8_RBC  ca_RBC kaNK_BW ka_RBC n_RBC nNK_BW;

tt=[1:size(oo_.irfs.dcNK_BW_eps,2)];

// Creating figures
// Figures 1 and 2
figure
subplot(3,2,1)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.tfp_eps,'k','LineWidth',5);
axis([0 20 0.9 1.5])
title('Permanent Shock: Productivity', 'HorizontalAlignment', 'center','FontName','Times New Roman', 'FontSize', 22);
subplot(3,2,2)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.tfp_eta,'k','LineWidth',5);
legend({'productivity'},'FontName','Times New Roman','FontSize',16,'Location','northeast')
axis([0 20 -0.1 1.1])
title('Transitory Shock: Productivity', 'HorizontalAlignment', 'center','FontName','Times New Roman', 'FontSize', 22);
subplot(3,2,3)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.rNK_BW_eps,'k','LineWidth',5);
axis([0 20 -0.2 1.1])
title('Permanent Shock: Real Interest Rate', 'HorizontalAlignment', 'center','FontName','Times New Roman', 'FontSize', 22);
subplot(3,2,4)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.rNK_BW_eta,'k','LineWidth',5);
legend({'real interest rate'},'FontName','Times New Roman','FontSize',16,'Location','northeast')
axis([0 20 -0.2 1.1])
title('Transitory Shock: Real Interest Rate', 'HorizontalAlignment', 'center','FontName','Times New Roman', 'FontSize', 22);
subplot(3,2,5)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.caNK_BW_eps,'k','LineWidth',5);
axis([0 20 0.2 1.5])
title('Permanent Shock: Consumption', 'HorizontalAlignment', 'center','FontName','Times New Roman','FontSize', 22);
subplot(3,2,6)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.caNK_BW_eta,'k','LineWidth',5);
legend({'consumption'},'FontName','Times New Roman','FontSize',16,'Location','northeast')
axis([0 20 -0.1 1.1])
title('Transitory Shock: Consumption', 'HorizontalAlignment', 'center','FontName','Times New Roman', 'FontSize', 22);

// Figure 3
figure
subplot(1,2,1)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.rNK_BW_eps,'k',tt,oo_.irfs.r_RBC_eps,'k--','LineWidth',5);
axis([0 20 -0.3 1.8])
title('Permanent Shock: Real Interest Rate', 'HorizontalAlignment', 'center', 'FontName','Times New Roman','FontSize', 22);
subplot(1,2,2)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.rNK_BW_eta,'k',tt,oo_.irfs.r_RBC_eta,'k--','LineWidth',5);
legend({'NK-BW model','Standard RBC model'},'FontName','Times New Roman','FontSize',16,'Location','southeast')
axis([0 20 -0.3 1.8])
title('Transitory Shock: Real Interest Rate', 'HorizontalAlignment', 'center','FontName','Times New Roman', 'FontSize', 22);

// Figure 4
figure
subplot(1,1,1)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.conditional_variance_decomposition(3,:,1),'k','LineWidth',5);
legend({'NK-BW model'},'FontName','Times New Roman','FontSize',16,'Location','southeast')
title('Permanent Shock: Output', 'HorizontalAlignment', 'center','FontName','Times New Roman', 'FontSize', 22);

// Figure 5
figure
subplot(2,1,1)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.yaNK_BW_eps,'k',tt,oo_.irfs.caNK_eps,'k--','LineWidth',5);
legend({'NK-BW model','Simple NK model'},'FontName','Times New Roman','FontSize',16,'Location','southeast')
title('Output', 'HorizontalAlignment', 'center', 'FontName','Times New Roman','FontSize', 22);
subplot(2,1,2)
set(gca, 'FontName', 'Times New Roman')
set(gca, 'FontSize', 16)
plot(tt,oo_.irfs.iaNK_BW_eps,'k','LineWidth',5);
legend({'NK-BW model'},'FontName','Times New Roman','FontSize',16,'Location','southeast')
title('Investment', 'HorizontalAlignment', 'center','FontName','Times New Roman', 'FontSize', 22);
